"""
    file        scrdr_tcp_client.py
    brief       SpringCore - SpringPark SCRDR client
    author      Matthieu Barreteau <matthieu.b@springcard.com>
    date        08/03/2021
 
    copyright   Copyright (c) 2021 SPRINGCARD SAS, FRANCE - www.springcard.com
    version     MBA 27/01/2021: initial release, plain connexion

    Requirements:
    - Python >= 3.7.3

    Purpose:
    The aim of this project is to create a simple tcp SCRDR client.

    You can stop this script using CTRL+C
    """

# libraries import
from threading import Thread, Event
import logging
import socket
import getopt
import sys
import os


# helpers import
from helpers.ctrlccatcher import CtrlCCatcher


# display_help function
def display_help( script_name ):

    print( script_name )
    print( "[--port=<port>]                 TCP port (default: 4000)" )
    print( "[--ip=<ip address>]             Server IP address" )
    print( "[--nolog]                       Silent mode" )
    print( "[--logtofile]                   Output to debug.log" )
    print( "[--error_at=<number>]           Force no response at packet number <number>" )
    sys.exit()


# main function  
def main( script_name, argv ):
    
    # default settings
    port = 4000
    use_log = 1
    server_address = ""

    # options parsing
    try:
        opts, _ = getopt.getopt(argv,"h",[ "help", "port=", "ip=", "nolog", "logtofile" ])
    except getopt.GetoptError as e:
        print(e)
        display_help( script_name )

    for opt, arg in opts:
        if opt in ( "-h", "--help"):
            display_help( script_name )
        elif opt == "--nolog":
            use_log = 0
        elif opt == "--logtofile":
            use_log = 2
        elif opt == "--ip":
            server_address = str(arg)
        elif opt == "--port":
            try:
                port = int(arg)
            except ValueError:
                print( "Port must be an integer number!" )
                sys.exit()

    if server_address == "":
        print( "Server IP address must be specified!" )
        sys.exit()       

    # set debug options
    log_level = logging.DEBUG 
    if use_log:
        if use_log > 1:
            logging.basicConfig(format='%(asctime)s - %(levelname)s - %(message)s', \
                                datefmt='%d-%b-%y %H:%M:%S', \
                                filename=f"debug.log", filemode="w", \
                                level=log_level)
        else:
            logging.basicConfig(format='%(asctime)s - %(levelname)s - %(message)s', \
                                datefmt='%d-%b-%y %H:%M:%S', \
                                level=log_level)  

    # CTRL+C catcher thread (this could be useful on some Windows platforms)
    ctrlccatch = CtrlCCatcher()
    ctrlccatch.start()

    # Create a TCP/IP socket
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)    
    sock.setblocking(0)
    sock.settimeout(0.5)

    # TCP mode, try to connect to our SCRDR server
    logging.info( f"connecting to {server_address} port {port}" )
    server_settings = ( server_address, port )        
    try:
        # try to connect to this server
        sock.connect( server_settings )

        # look for CTRL+C
        while ctrlccatch.wait():
            try:
                # wait for new data
                data, address = sock.recvfrom(512)
                if data:
                    data = data.decode( "ascii" )
                    logging.info( data )
            except socket.timeout:
                pass
    
    except socket.timeout:
        logging.info( "Can't connect to this server..." )

    # stop all process and exit
    logging.info( "Cleaning..." )
    ctrlccatch.terminate()
    ctrlccatch.join()
    logging.info( "Done." )


# entry point
if __name__ == "__main__":
    # add "this script" folder to the library search path
    # could be useful on Windows platform
    sys.path.append( os.path.dirname( os.path.realpath( __file__ ) ) )
  
    # start our application
    main( os.path.basename( __file__ ), sys.argv[1:] )
    
# EOF